==> dmail-1.14.tgz: dmail/sendmail.c <==

/*
 *  SENDMAIL.C
 *
 *  Global Routines:    DO_REPLY()
 *                      DO_MAIL()
 *
 *  Static Routines:    WORD_SIZE()
 *                      FOPEN_SCRATCH()
 *                      FREOPEN_SCRATCH()
 *                      FCLOSE_SCRATCH()
 *                      FTERMINATE_SCRATCH()
 *                      DELETE_SCRATCH()
 *                      RUN_VI()
 *                      SEND_MAIL() 
 *
 *
 */

#include "dmail.h"

Prototype int do_reply(char *garbage, int itext);
Prototype int do_ask(char *garbage, int com);

void copy_header(FILE *fi, char *varName);
void fopen_scratch(void);
void fclose_scratch(void);
int word_size(const char *str);
int run_vi(void);
void send_mail(void);
int archive_mail(char *ptr);
char *indexFieldStr(const char *fields, int index);


FILE *fi;
char file[64];

char *indexFieldStr(const char *fields, int index);

int
do_reply(char *garbage, int itext)
{
    int i, j;
    volatile int anyargs  = 0;
    int to_field, cc_field;
    char * volatile istr;

    if (!(istr = get_var(LEVEL_SET, "_headchar")))
        istr = ">";
    if (push_base()) {
        push_break();
        pop_base();
        fclose_scratch();
        puts ("ABORTED, no mail sent");
		unlink(file);
        pop_break();
        return (-1);
    }
    fopen_scratch();

    fprintf(fi, "To: ");

    for (i = 1; i < ac; ++i) {
        if (*av[i] >= '0'  &&  *av[i] <= '9') {
            if ((j = indexof(atoi(av[i]))) < 0) {
                puts ("No such message");
                fclose_scratch();
		unlink(file);
                pop_break();
                return (-1);
            }
            Current = j;
        } else {
            if (anyargs)
		fprintf(fi, ", ");
            anyargs = 1;
	    fprintf(fi, "%s", av[i]);
        }
    }

    switch (itext) {
    case R_FORWARD:
	fprintf(fi, "\n");
	fprintf(fi, "Subject: \n");
        break;
    case R_INCLUDE:
    case R_REPLY:

        /*
         * Calculate remainder of To: field
         */

        if (Current >= 0) {
            char *rf;

            /*
             * Obtain field to use in reply
             */

            if ((rf = get_var(LEVEL_SET, "replyfields")) != NULL) {
		int i;
		char *idx;

		for (i = 0; (idx = indexFieldStr(rf, i)) != NULL; ++i) {
		    int flen;
		    const char *p = get_field(idx, &flen);

		    if (*p) {
			fprintf(fi, "%s", (anyargs ? ", " : ""));
			fwrite(p, flen, 1, fi);
			anyargs = 1;
		        break;
		    }
		}
		if (idx == NULL)
		    rf = NULL;
	    }
	    if (rf == NULL) {
		fprintf(fi, "%s", (anyargs ? ", " : ""));
		fwrite(Entry[Current].from, word_size(Entry[Current].from), 1, fi);
		anyargs = 1;
	    }
        }
	fprintf(fi, "\n");

        /*
         * Calculate Cc: field
         */

	anyargs = 0;
	fprintf(fi, "Cc: ");

	{
	    int flen;
	    const char *p = get_field("To:", &flen);

	    if (flen > 0 && p[flen-1] == '\n')
		--flen;

	    to_field = (*p && flen > 0) ? 1 : 0;

	    if (flen > 0) {
		fwrite(p, flen, 1, fi);
		anyargs = 1;
	    }
	}

	{
	    int flen;
	    const char *p = get_field("Cc:", &flen);

	    if (flen > 0 && p[flen-1] == '\n')
		--flen;

	    cc_field = (*p && flen > 0) ? 1 : 0;
	    if (flen > 0) {
		fprintf(fi, "%s", (anyargs ? ", " : ""));
		fwrite(p, flen, 1, fi);
		anyargs = 1;
	    }
	}
        fputs("\n", fi);

        /*
         * Subject:
         */

	fprintf(fi, "Subject: ");
	{
	    int flen;
	    const char *p = get_field("Subject:", &flen);

	    if (flen >= 3 && strncmp(p, "Re:", 3) != 0)
		fprintf(fi, "Re: ");
	    fwrite(p, flen, 1, fi);
	    fprintf(fi, "\n");
	}

        /*
         * Additional headers to copy into reply
         */

        {
            char *rh;
            char *idx;
            int i;

            if ((rh = get_var(LEVEL_SET, "copyheaders")) != NULL) {
		for (i = 0; (idx = indexFieldStr(rh, i)) != NULL; ++i) {
		    int flen;
		    const char *p = get_field(idx, &flen);

		    fprintf(fi, "%s ", idx);
		    fwrite(p, flen, 1, fi);
		    fprintf(fi, "\n");
		}
	    }
	}
        break;
    case R_MAIL:
	fprintf(fi, "\n");
        {
            char *rh;
            int i;

	    if ((rh = get_var(LEVEL_SET, "mailheaders")) != NULL) {
	        if (strstr(rh, "Subject:") != NULL)
		    fputs ("Subject: \n", fi);
		for (i = 0; indexFieldStr(rh, i); ++i) {
		    fprintf(fi, "%s: \n", indexFieldStr(rh, i));
		}
	    } else {
		fputs ("Subject: \n", fi);
	    }
	}
        break;
    default:
        puts ("INTERNAL STUPID MAIL ERROR: REPLY");
        break;
    }
    copy_header (fi, "headfile");
    fputc('\n', fi);
    fputc('\n', fi);
    if (itext == R_FORWARD  || itext == R_FWDINCL || itext == R_INCLUDE) {
	int pos = Entry[Current].fpos;
	int inHeaders = 2;

	if (itext == R_FORWARD || itext == R_FWDINCL)
	    fprintf(fi, "ORIGINALLY ");

	do {
	    if (FileData[pos] == '\n')
		inHeaders = 0;

	    if (inHeaders == 0 || itext == R_FORWARD || itext == R_FWDINCL) {
		int l = data_line_length(FileData + pos);
		if (itext == R_INCLUDE || itext == R_FWDINCL)
		    fputs(istr, fi);
		fwrite(FileData + pos, l, 1, fi);
	    }
	    if (position_next(&pos) <= 0)
		break;
	} while (!position_at_from(pos));
	fputc('\n', fi);
    }
    copy_header (fi, "sigfile");
    fclose_scratch();
    if (itext != R_MAIL) {
        push_break();
        if (Current >= 0) {
            Entry[Current].status |= ST_SCR;
            write_file("#", O_CREAT | O_TRUNC, ST_SCR, 0);
            Entry[Current].status &= ~ST_SCR;
        }
        pop_break();
    }
    j = -1;
loop:
    ++j;
    if (run_vi() || j) {
        push_break();
        switch (do_ask(NULL, 0)) {
        case 1:
            puts ("SENDING.. wait");
            send_mail();
            {
                FILE *li = fopen(file, "r");
                char buf[128], *ptr = NULL;

                if (li) {
                    while (fgets(buf, 128, li) && buf[0] != '\n') {
                        if (strncmp(buf, "Farchive:", 9) == 0) {
                            buf[strlen(buf)-1] = '\0';
                            for (ptr = buf + 9; *ptr == ' '; ++ptr);
                            if (ptr[0] == '$') 
                                ptr = get_var(LEVEL_SET, ptr+1);
                            break;
                        }
                    }
                    fclose(li);
                }
                archive_mail(ptr);
            }
            unlink(file);
            break;
        case 2:
            pop_break();
            goto loop;
        default:
            unlink (file);
            break;
        }
        pop_base();
        pop_break();
    } else {
        puts ("File not modified or ABORTED, no mail sent");
        unlink(file);
        pop_base();
    }
    unlink ("#");
    return(1);
}

int
do_ask(char *garbage, int com)
{
    char in[256];

    if (!S_ask)
        return (1);
    fputs ("\n(Send, Vi, Quit) ?", stdout);
    fflush(stdout);
    if (fgets(in, sizeof(in), stdin) == NULL)
	return(1);

    switch (in[0]) {
    case 's':
    case 'S':
        return (1);
    case 'q':
    case 'Q':
        puts ("ABORT, no mail sent");
        return (3);
    case 'v':
    case 'V':
    default:
        return (2);
    }
}

void
copy_header(FILE *fi, char *varName)
{
    FILE *fs;
    char *ptr;

    if ((ptr = get_var (LEVEL_SET, varName)) != NULL) {
        push_break();
        if ((fs = fopen (ptr, "r")) != NULL) {
            while (fgets (Buf, MAXFIELDSIZE, fs) != NULL)
                fputs (Buf, fi);
            fclose (fs);
        } else {
            printf ("Cannot open header file %d %s\n", strlen(ptr), ptr);
            perror ("fopen");
        }
        pop_break();
    }
}

void
fopen_scratch(void)
{
    static int c;
    int fd;

    sprintf(file, "/var/tmp/dmt%d%d", (int)getpid(), c++);
    fd = open(file, O_RDWR|O_CREAT|O_TRUNC, 0700);
    if (fd < 0) {
        perror ("Dmail, cannot open scratch file");
        done (1);
    }
    fi = fdopen(fd, "w+");
}

void
fclose_scratch(void)
{
    if (fi != NULL) {
        fflush (fi);
        fclose (fi);
        fi = NULL;
    }
}


int
word_size(const char *str)
{
    int size = 0;

    while (*str) {
        if (*str == ' ')
            return (size);
        ++str;
        ++size;
    }
    return (size);
}

int
run_vi(void)
{
    char buf[64];
    volatile int ret, pid = 0;
    struct stat stat1, stat2;
    char *argv[3];

    argv[0] = visual;
    argv[1] = file;
    argv[2] = NULL;
    if (push_base()) {
        push_break();
        pop_base();
        if (pid) {
            kill (pid, SIGKILL);
            sprintf (buf, "/var/tmp/Ex%d", pid); unlink (buf);
            sprintf (buf, "/var/tmp/Rx%d", pid); unlink (buf);
            wait(0);
            system ("clear; reset ; clear");
            pid = 0;
        }
        pop_break();
        return (0);
    }
    stat1.st_mtime = stat2.st_mtime = stat1.st_ctime = stat2.st_ctime = 0;
    stat (file, &stat1);
    if (S_novibreak)
        push_break();

    pid = vfork();
    if (!pid) {
        execv (visual, argv);
    printf ("Cannot exec visual: %s\n", visual);
        _exit (1);
    }
    while ((ret = wait(0)) > 0) {
        if (ret == pid)
            break;
    }
    if (S_novibreak)
        pop_break();
    stat (file, &stat2);
    pop_base();
    return (!(stat1.st_mtime==stat2.st_mtime));
}

void
send_mail(void)
{
    int fd, stdin_fd;
    char *argv[6];

    push_break();
    argv[0] = S_sendmail;
    argv[1] = "-t";
    argv[2] = "-oem";
    argv[3] = "-i";
    if (S_verbose) {
        argv[4] = "-v";
        argv[5] = NULL;
    } else {
        argv[4] = NULL;
    }
    fd = open (file, O_RDONLY, 0);
    if (fd < 0) {
        perror ("Dmail, Cannot open scratch file");
        done (1);
    }
    lseek(fd, 0, 0);
    stdin_fd = dup (0);
    dup2 (fd, 0);       /* STDIN = message file */
    close(fd);          /* don't need message file anymore  */
    if (!fork()) {
        int fd = open("/dev/tty", O_RDWR, 0);
        if (fd >= 0) {
            ioctl(fd, TIOCNOTTY, 0);
#ifdef sgi
	    setpgrp();
#else
            setpgrp(0, 0);
#endif
            close(fd);
            freopen("/dev/null", "w", stdout);
            freopen("/dev/null", "w", stderr);
        }
        execv (S_sendmail, argv);
        printf ("Unable to exec sendmail: %s\n", S_sendmail);
        _exit (1);
    }
    dup2 (stdin_fd, 0);     /* restore STDIN    */
    close(stdin_fd);
    if (S_verbose) {
        puts ("Waiting for sendmail...");
        wait (0);
        puts ("Sendmail done");
    }
    pop_break();
}

int
archive_mail(char *ptr)
{
    FILE *ifi, *ofi;
    long tim = time(NULL);

    if (!ptr)
        ptr = get_var(LEVEL_SET, "archive");
    if (ptr == NULL || *ptr == '\0')
        return(-1);
    ifi = fopen(file, "r");
    if (ifi == NULL) {
        puts ("Cannot open scratch file");
        return(-1);
    }
    ofi = fopen(ptr, "a");
    if (ofi == NULL) {
        puts ("Cannot open archive file");
        fclose(ifi);
        return(-1);
    }
    sprintf (Buf, "\nFrom %s (ARCHIVE)\n", user_name);
    fputs (Buf, ofi);
    sprintf (Buf, "Date: %s", ctime(&tim));
    fputs (Buf, ofi);
    while (fgets (Buf, MAXFIELDSIZE, ifi)) 
        fputs (Buf, ofi);
    fclose(ofi);
    fclose(ifi);
    return (1);
}

char *
indexFieldStr(const char *fields, int index)
{
    const char *ptr = fields;
    static char buf[256];
    int i = 0;

    while (index-- >= 0) {
    	ptr += i;
	while (*ptr == ' ' || *ptr == '\t') {
	    ++ptr;
	}
	for (i = 0; ptr[i] && ptr[i] != ':'; ++i)
	    ;
	if (ptr[i] != ':') {
	    i = 0;
	    break;
	}
	++i;
    }
    memcpy(buf, ptr, i);
    buf[i] = 0;
    return((buf[0]) ? buf : NULL);
}

==> dmail-1.18.tgz: dmail/sendmail.c <==

/*
 *  SENDMAIL.C
 *
 *  Global Routines:    DO_REPLY()
 *                      DO_MAIL()
 *
 *  Static Routines:    WORD_SIZE()
 *                      FOPEN_SCRATCH()
 *                      FREOPEN_SCRATCH()
 *                      FCLOSE_SCRATCH()
 *                      FTERMINATE_SCRATCH()
 *                      DELETE_SCRATCH()
 *                      RUN_VI()
 *                      SEND_MAIL() 
 *
 *
 */

#include "dmail.h"

Prototype int do_reply(char *garbage, int itext);
Prototype int do_ask(char *garbage, int com);

void copy_header(FILE *fi, char *varName);
void fopen_scratch(void);
void fclose_scratch(void);
int word_size(const char *str);
int run_vi(void);
void send_mail(void);
int archive_mail(char *ptr);
char *indexFieldStr(const char *fields, int index);


FILE *fi;
char file[64];

char *indexFieldStr(const char *fields, int index);

int
do_reply(char *garbage, int itext)
{
    int i, j;
    volatile int anyargs  = 0;
    int to_field, cc_field;
    char * volatile istr;

    if (!(istr = get_var(LEVEL_SET, "_headchar")))
        istr = ">";
    if (push_base()) {
        push_break();
        pop_base();
        fclose_scratch();
        puts ("ABORTED, no mail sent");
		unlink(file);
        pop_break();
        return (-1);
    }
    fopen_scratch();

    fprintf(fi, "To: ");

    for (i = 1; i < ac; ++i) {
        if (*av[i] >= '0'  &&  *av[i] <= '9') {
            if ((j = indexof(atoi(av[i]))) < 0) {
                puts ("No such message");
                fclose_scratch();
		unlink(file);
                pop_break();
                return (-1);
            }
            Current = j;
        } else {
            if (anyargs)
		fprintf(fi, ", ");
            anyargs = 1;
	    fprintf(fi, "%s", av[i]);
        }
    }

    switch (itext) {
    case R_FORWARD:
	fprintf(fi, "\n");
	fprintf(fi, "Subject: \n");
        break;
    case R_INCLUDE:
    case R_REPLY:

        /*
         * Calculate remainder of To: field
         */

        if (Current >= 0) {
            char *rf;

            /*
             * Obtain field to use in reply
             */

            if ((rf = get_var(LEVEL_SET, "replyfields")) != NULL) {
		int i;
		char *idx;

		for (i = 0; (idx = indexFieldStr(rf, i)) != NULL; ++i) {
		    int flen;
		    const char *p = get_field(idx, &flen);

		    if (*p) {
			fprintf(fi, "%s", (anyargs ? ", " : ""));
			fwrite(p, flen, 1, fi);
			anyargs = 1;
		        break;
		    }
		}
		if (idx == NULL)
		    rf = NULL;
	    }
	    if (rf == NULL) {
		fprintf(fi, "%s", (anyargs ? ", " : ""));
		fwrite(Entry[Current].from, word_size(Entry[Current].from), 1, fi);
		anyargs = 1;
	    }
        }
	fprintf(fi, "\n");

        /*
         * Calculate Cc: field
         */

	anyargs = 0;
	fprintf(fi, "Cc: ");

	{
	    int flen;
	    const char *p = get_field("To:", &flen);

	    if (flen > 0 && p[flen-1] == '\n')
		--flen;

	    to_field = (*p && flen > 0) ? 1 : 0;

	    if (flen > 0) {
		fwrite(p, flen, 1, fi);
		anyargs = 1;
	    }
	}

	{
	    int flen;
	    const char *p = get_field("Cc:", &flen);

	    if (flen > 0 && p[flen-1] == '\n')
		--flen;

	    cc_field = (*p && flen > 0) ? 1 : 0;
	    if (flen > 0) {
		fprintf(fi, "%s", (anyargs ? ", " : ""));
		fwrite(p, flen, 1, fi);
		anyargs = 1;
	    }
	}
        fputs("\n", fi);

        /*
         * Subject:
         */

	fprintf(fi, "Subject: ");
	{
	    int flen;
	    const char *p = get_field("Subject:", &flen);

	    if (flen >= 3 && strncmp(p, "Re:", 3) != 0)
		fprintf(fi, "Re: ");
	    fwrite(p, flen, 1, fi);
	    fprintf(fi, "\n");
	}

        /*
         * References:
         */

	fprintf(fi, "References: ");
	{
	    int flen;
	    const char *p = get_field("References:", &flen);

	    if (p) {
		fwrite(p, flen, 1, fi);
		fprintf(fi, " ");
	    }
	    p = get_field("Message-ID:", &flen);
	    if (p)
		fwrite(p, flen, 1, fi);
	    else
		fprintf(fi, "<non-conformant-source@nowhere.at.the.net>");
	    fprintf(fi, "\n");
	}

        /*
         * Additional headers to copy into reply
         */

        {
            char *rh;
            char *idx;
            int i;

            if ((rh = get_var(LEVEL_SET, "copyheaders")) != NULL) {
		for (i = 0; (idx = indexFieldStr(rh, i)) != NULL; ++i) {
		    int flen;
		    const char *p = get_field(idx, &flen);

		    fprintf(fi, "%s ", idx);
		    fwrite(p, flen, 1, fi);
		    fprintf(fi, "\n");
		}
	    }
	}
        break;
    case R_MAIL:
	fprintf(fi, "\n");
        {
            char *rh;
            int i;

	    if ((rh = get_var(LEVEL_SET, "mailheaders")) != NULL) {
	        if (strstr(rh, "Subject:") != NULL)
		    fputs ("Subject: \n", fi);
		for (i = 0; indexFieldStr(rh, i); ++i) {
		    fprintf(fi, "%s: \n", indexFieldStr(rh, i));
		}
	    } else {
		fputs ("Subject: \n", fi);
	    }
	}
        break;
    default:
        puts ("INTERNAL STUPID MAIL ERROR: REPLY");
        break;
    }
    copy_header (fi, "headfile");
    fputc('\n', fi);
    fputc('\n', fi);
    if (itext == R_FORWARD  || itext == R_FWDINCL || itext == R_INCLUDE) {
	int pos = Entry[Current].fpos;
	int inHeaders = 2;

	if (itext == R_FORWARD || itext == R_FWDINCL)
	    fprintf(fi, "ORIGINALLY ");

	do {
	    if (FileData[pos] == '\n')
		inHeaders = 0;

	    if (inHeaders == 0 || itext == R_FORWARD || itext == R_FWDINCL) {
		int l = data_line_length(FileData + pos, 0);
		if (itext == R_INCLUDE || itext == R_FWDINCL)
		    fputs(istr, fi);
		fwrite(FileData + pos, l, 1, fi);
	    }
	    if (position_next(&pos) <= 0)
		break;
	} while (!position_at_from(pos));
	fputc('\n', fi);
    }
    copy_header (fi, "sigfile");
    fclose_scratch();
    if (itext != R_MAIL) {
        push_break();
        if (Current >= 0) {
            Entry[Current].status |= ST_SCR;
            write_file("#", O_CREAT | O_TRUNC, ST_SCR, 0);
            Entry[Current].status &= ~ST_SCR;
        }
        pop_break();
    }
    j = -1;
loop:
    ++j;
    if (run_vi() || j) {
        push_break();
        switch (do_ask(NULL, 0)) {
        case 1:
            puts ("SENDING.. wait");
            send_mail();
            {
                FILE *li = fopen(file, "r");
                char buf[128], *ptr = NULL;

                if (li) {
                    while (fgets(buf, 128, li) && buf[0] != '\n') {
                        if (strncmp(buf, "Farchive:", 9) == 0) {
                            buf[strlen(buf)-1] = '\0';
                            for (ptr = buf + 9; *ptr == ' '; ++ptr);
                            if (ptr[0] == '$') 
                                ptr = get_var(LEVEL_SET, ptr+1);
                            break;
                        }
                    }
                    fclose(li);
                }
                archive_mail(ptr);
            }
            unlink(file);
            break;
        case 2:
            pop_break();
            goto loop;
        default:
            unlink (file);
            break;
        }
        pop_base();
        pop_break();
    } else {
        puts ("File not modified or ABORTED, no mail sent");
        unlink(file);
        pop_base();
    }
    unlink ("#");
    return(1);
}

int
do_ask(char *garbage, int com)
{
    char in[256];

    if (!S_ask)
        return (1);
    fputs ("\n(Send, Vi, Quit) ?", stdout);
    fflush(stdout);
    if (fgets(in, sizeof(in), stdin) == NULL)
	return(1);

    switch (in[0]) {
    case 's':
    case 'S':
        return (1);
    case 'q':
    case 'Q':
        puts ("ABORT, no mail sent");
        return (3);
    case 'v':
    case 'V':
    default:
        return (2);
    }
}

void
copy_header(FILE *fi, char *varName)
{
    FILE *fs;
    char *ptr;

    if ((ptr = get_var (LEVEL_SET, varName)) != NULL) {
        push_break();
        if ((fs = fopen (ptr, "r")) != NULL) {
            while (fgets (Buf, MAXFIELDSIZE, fs) != NULL)
                fputs (Buf, fi);
            fclose (fs);
        } else {
            printf ("Cannot open header file %d %s\n", strlen(ptr), ptr);
            perror ("fopen");
        }
        pop_break();
    }
}

void
fopen_scratch(void)
{
    static int c;
    int fd;

    sprintf(file, "/var/tmp/dmt%d%d", (int)getpid(), c++);
    fd = open(file, O_RDWR|O_CREAT|O_TRUNC, 0700);
    if (fd < 0) {
        perror ("Dmail, cannot open scratch file");
        done (1);
    }
    fi = fdopen(fd, "w+");
}

void
fclose_scratch(void)
{
    if (fi != NULL) {
        fflush (fi);
        fclose (fi);
        fi = NULL;
    }
}


int
word_size(const char *str)
{
    int size = 0;

    while (*str) {
        if (*str == ' ')
            return (size);
        ++str;
        ++size;
    }
    return (size);
}

int
run_vi(void)
{
    char buf[64];
    volatile int ret, pid = 0;
    struct stat stat1, stat2;
    char *argv[3];

    argv[0] = visual;
    argv[1] = file;
    argv[2] = NULL;
    if (push_base()) {
        push_break();
        pop_base();
        if (pid) {
            kill (pid, SIGKILL);
            sprintf (buf, "/var/tmp/Ex%d", pid); unlink (buf);
            sprintf (buf, "/var/tmp/Rx%d", pid); unlink (buf);
            wait(0);
            system ("clear; reset ; clear");
            pid = 0;
        }
        pop_break();
        return (0);
    }
    stat1.st_mtime = stat2.st_mtime = stat1.st_ctime = stat2.st_ctime = 0;
    stat (file, &stat1);
    if (S_novibreak)
        push_break();

    pid = vfork();
    if (!pid) {
        execv (visual, argv);
    printf ("Cannot exec visual: %s\n", visual);
        _exit (1);
    }
    while ((ret = wait(0)) > 0) {
        if (ret == pid)
            break;
    }
    if (S_novibreak)
        pop_break();
    stat (file, &stat2);
    pop_base();
    return (!(stat1.st_mtime==stat2.st_mtime));
}

void
send_mail(void)
{
    int fd, stdin_fd;
    char *argv[6];

    push_break();
    argv[0] = S_sendmail;
    argv[1] = "-t";
    argv[2] = "-oem";
    argv[3] = "-i";
    if (S_verbose) {
        argv[4] = "-v";
        argv[5] = NULL;
    } else {
        argv[4] = NULL;
    }
    fd = open (file, O_RDONLY, 0);
    if (fd < 0) {
        perror ("Dmail, Cannot open scratch file");
        done (1);
    }
    lseek(fd, 0, 0);
    stdin_fd = dup (0);
    dup2 (fd, 0);       /* STDIN = message file */
    close(fd);          /* don't need message file anymore  */
    if (!fork()) {
        int fd = open("/dev/tty", O_RDWR, 0);
        if (fd >= 0) {
            ioctl(fd, TIOCNOTTY, 0);
#ifdef sgi
	    setpgrp();
#else
            setpgrp(0, 0);
#endif
            close(fd);
            freopen("/dev/null", "w", stdout);
            freopen("/dev/null", "w", stderr);
        }
        execv (S_sendmail, argv);
        printf ("Unable to exec sendmail: %s\n", S_sendmail);
        _exit (1);
    }
    dup2 (stdin_fd, 0);     /* restore STDIN    */
    close(stdin_fd);
    if (S_verbose) {
        puts ("Waiting for sendmail...");
        wait (0);
        puts ("Sendmail done");
    }
    pop_break();
}

int
archive_mail(char *ptr)
{
    FILE *ifi, *ofi;
    long tim = time(NULL);

    if (!ptr)
        ptr = get_var(LEVEL_SET, "archive");
    if (ptr == NULL || *ptr == '\0')
        return(-1);
    ifi = fopen(file, "r");
    if (ifi == NULL) {
        puts ("Cannot open scratch file");
        return(-1);
    }
    ofi = fopen(ptr, "a");
    if (ofi == NULL) {
        puts ("Cannot open archive file");
        fclose(ifi);
        return(-1);
    }
    sprintf (Buf, "\nFrom %s (ARCHIVE)\n", user_name);
    fputs (Buf, ofi);
    sprintf (Buf, "Date: %s", ctime(&tim));
    fputs (Buf, ofi);
    while (fgets (Buf, MAXFIELDSIZE, ifi)) 
        fputs (Buf, ofi);
    fclose(ofi);
    fclose(ifi);
    return (1);
}

char *
indexFieldStr(const char *fields, int index)
{
    const char *ptr = fields;
    static char buf[256];
    int i = 0;

    while (index-- >= 0) {
    	ptr += i;
	while (*ptr == ' ' || *ptr == '\t') {
	    ++ptr;
	}
	for (i = 0; ptr[i] && ptr[i] != ':'; ++i)
	    ;
	if (ptr[i] != ':') {
	    i = 0;
	    break;
	}
	++i;
    }
    memcpy(buf, ptr, i);
    buf[i] = 0;
    return((buf[0]) ? buf : NULL);
}
